/*----------------------------------------------------------------------------
 Pendula - networked harmonograph and interval guessing game
 
 Copyright (c) 2010 Elie El Noune
 All rights reserved.
 http://ccrma.stanford.edu/~noune/
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 U.S.A.
 -----------------------------------------------------------------------------*/


//-----------------------------------------------------------------------------
// name: pendula.cpp
// 
// author: Elie El Noune
// libraries:
//          (OscPack) Ross Becina
//			(RtAudio) Gary Scavone
// Includes code from: Calories, oscpack examples
// Project for Stanford University | Music 256a: Music, Computing, and Design
//     http://ccrma.stanford.edu/courses/256a/
//-----------------------------------------------------------------------------

#include "DelayLine.h"

#include <assert.h>
#include <math.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <iostream>

void DelayLine::setDelayLen(int delayLen)
{

	m_delayLen = delayLen;
	//a  note was hit!!
	m_write = 0;
	
	assert( m_delayLen <= m_delayMax );
	assert( m_delayMax > 0 );
	assert( m_delayLen > 0 );
	m_read = m_write - m_delayLen;
	
	if (m_read < 0) {
		m_read += m_delayMax;
		
	}
		
}

void DelayLine::fillNoise()
{
	for (int i=0; i<m_delayMax; i++)
	{
		m_delay[i] = 2.0 * (SAMPLE)rand() / ((SAMPLE)RAND_MAX )- 1.0;
		
	}
    
}


//constructor
DelayLine::DelayLine(int delayMax, int delayLen)
{
	m_delayMax = delayMax;
	m_delayLen = delayLen;
	
	assert( m_delayLen <= m_delayMax );
	assert( m_delayMax > 0 );
	assert( m_delayLen > 0 );
	
	m_write = 0;
	m_read = m_write - m_delayLen;
	
	if (m_read < 0) {
		m_read += m_delayLen;
	
	}//m_read will always be = 0!!!
		
	//create the array
	m_delay = new SAMPLE[m_delayMax];
	memset( m_delay, 0, m_delayMax * sizeof(SAMPLE));
	
}

DelayLine::~DelayLine()
{
	delete [] m_delay;
	m_delay = NULL;
    
}

void DelayLine::write(SAMPLE input)
{
	m_delay[m_write] = input;
}


void DelayLine::increment()
{
	m_write++;
	m_read++;
	
	m_write %= m_delayMax;
	m_read %= m_delayMax;
	
}

SAMPLE DelayLine::read()
{
	return m_delay[m_read];
}

SAMPLE DelayLine::tick(SAMPLE input)
{
	SAMPLE output = read();
	write(input);
	increment();
	return output;
}

