/*----------------------------------------------------------------------------
 Pendula - networked harmonograph and interval guessing game
 
 Copyright (c) 2010 Elie El Noune
 All rights reserved.
 http://ccrma.stanford.edu/~noune/
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 U.S.A.
 -----------------------------------------------------------------------------*/


//-----------------------------------------------------------------------------
// name: Interval.h
// 
// author: Elie El Noune
// libraries:
//          (OscPack) Ross Becina
//			(RtAudio) Gary Scavone
// Includes code from: Calories, oscpack examples
// Project for Stanford University | Music 256a: Music, Computing, and Design
//     http://ccrma.stanford.edu/courses/256a/
//-----------------------------------------------------------------------------


#ifndef __INTERVAL__

#include <string>

using namespace std;

class interval {
	
	
public:
	
	interval() : num(1), higher(1), lower(1), number("unison"),quality("perfect") {};
	
	interval( int n, string q) { number = n; quality = q;}
	
	interval(int n) { compute(n+1, 1);}
	
	string getNumber() { return number;}
	string getQuality() {return quality; }
	string getInterval() {return quality+" "+number;}
	int compute(int a, int b)
	{
		if (a == 0 || b == 0)
		{
			num = 0; 
			number = "Unison";
			quality = "Perfect";
			higher = 1;
			lower = 1;
			return 0;
		}
		
		int diff = 0; 
		if (a <= b)
			diff = b-a;
		else
			diff = a-b;
		
		if (diff == 0)
		{
		  num = 0; 
		  number = "Unison";
		  quality = "Perfect";
			higher = 1.001;
			lower = 0.999;
		  
		  return 1;
		}	
			
		int halfsteps = diff % 12;
		
		switch( halfsteps )
		{
			case (0):
				num = 12;
				number = "Octave";
				quality = "Perfect";
				higher = 2.001;
				lower = 0.999;
				
				break;
		
			case (1):
				num = 1;
				number = "Second";
				quality = "Minor";
				//higher = 16;
				//lower = 15;
				higher = 16.001;
				lower = 14.999;
				break;
				
			case (2):
				num = 2;
				number = "Second";
				quality = "Major";
				higher = 9.001;
				lower = 8.001;
				//higher = 1.125;
				//lower = 1;
				break;
				
			case (3):
				num = 3;
				number = "Third";
				quality = "Minor";
				higher = 6.001;
				lower = 4.999;
				//higher = 1.2;
				//lower = 1;
				break;
				
			case (4):
				num = 4;
				number = "Third";
				quality = "Major";
				higher = 5.001;
				lower = 4.001;
				//higher = 1.25;
				//lower = 1;
				break;
				
			case (5):
				num = 5;
				number = "Fourth";
				quality = "Perfect";
				//higher = 4;
				//lower = 3;
				higher = 0.801;
				lower = 0.601;
				break;
				
			case (6):
				num = 6;
				number = "";
				quality = "Tritone";
				//higher = 45;
				//lower = 32;
				higher = 22.5001;
				lower = 16.0001;
				break;
				
			case (7):
				num = 7;
				number = "Fifth";
				quality = "Perfect";
				//higher = 3;
				//lower = 2;
				higher = 1.201;
				lower = 0.801;
				break;
				
			case (8):
				num = 8;
				number = "Sixth";
				quality = "Minor";
				//higher = 8;
				//lower = 5;
				higher = 1.001;
				lower = 1.601;
				break;
				
			case (9):
				num = 9;
				number = "Sixth";
				quality = "Major";
				//higher = 5;
				//lower = 3;
				higher = 2.001;
				lower = 1.201;
				break;
				
			case (10):
				num = 10;
				number = "Seventh";
				quality = "Minor";
				//higher = 9;
				//lower = 5;
				higher = 1.801;
				lower = 1.001;
				break;
				
			case (11):
				num = 11;
				number = "Seventh";
				quality = "Major";
				//higher = 15;
				//lower = 8;
				higher = 7.501;
				lower = 4.001;
				break;
		}
		//sucess
		return 1;
	}
	
	double getLower() {return lower;}
	double getHigher() {return higher;}
	int getNum() {return num; }
	int setInterval(int n) { return compute(n+1, 1);}
private:
	
	int num;
	string number;
	string quality;
	double higher;
	double lower;
	
};


#endif
