/*----------------------------------------------------------------------------
 Pendula - networked harmonograph and interval guessing game
 
 Copyright (c) 2010 Elie El Noune
 All rights reserved.
 http://ccrma.stanford.edu/~noune/
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 U.S.A.
 -----------------------------------------------------------------------------*/


//-----------------------------------------------------------------------------
// name: KarplusSynth.h
// 
// author: Elie El Noune
// libraries:
//          (OscPack) Ross Becina
//			(RtAudio) Gary Scavone
// Includes code from: Calories, oscpack examples
// Project for Stanford University | Music 256a: Music, Computing, and Design
//     http://ccrma.stanford.edu/courses/256a/
//-----------------------------------------------------------------------------


#ifndef __KARPLUS_SYNTH__

#include "defs.h"
#include "DelayLine.h"
#include "OneZero.h"

class KarplusSynth
{
public:
    
    // Constructor
    KarplusSynth(){
		delayLine = new DelayLine(SRATE, 100);
		onezero = new OneZero(0.5, 0.5);
		
	}
    
    // Destructor
    ~KarplusSynth();
    
    // Fill the buffer with audio
    void fillBuffer( void * buffer, int buffer_size ){
		
		SAMPLE * buffy = (SAMPLE *) buffer;
		
		for (int i=0; i<buffer_size; i++)
		{
			SAMPLE output = delayLine->read();
			SAMPLE smoothedOutput = onezero->tick(output);
			buffy[i] += delayLine->tick(0.997*smoothedOutput);
		}
		
	}
    
    // Trigger note on
    void noteOn(SAMPLE velocity, SAMPLE freq){
		delayLine->setDelayLen(SRATE/freq);
		delayLine->fillNoise();
	}
    
private:
    DelayLine *delayLine;
	OneZero *onezero;

};




#endif