/*----------------------------------------------------------------------------
 Pendula - networked harmonograph and interval guessing game
 
 Copyright (c) 2010 Elie El Noune
 All rights reserved.
 http://ccrma.stanford.edu/~noune/
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 U.S.A.
 -----------------------------------------------------------------------------*/


//-----------------------------------------------------------------------------
// name: pendula.cpp
// 
// author: Elie El Noune
// libraries:
//          (OscPack) Ross Becina
//			(RtAudio) Gary Scavone
// Includes code from: Calories, oscpack examples
// Project for Stanford University | Music 256a: Music, Computing, and Design
//     http://ccrma.stanford.edu/courses/256a/
//-----------------------------------------------------------------------------


#ifndef __MIDI_EVENT__

#include <math.h>

class MidiEvent{

public:

	//constructor
	
	MidiEvent() {};
	
	MidiEvent(double stamp, std::vector <unsigned char> * message)
	{
		//assuming size is always 3 bytes
		m_stamp = stamp; 
		m_status = (int)message->at(0);
		m_note = (int)message->at(1);
		m_velocity = (int)message->at(2);
			
	}
	
	MidiEvent(double stamp, int status, int note, int velocity)
	{
		//assuming size is always 3 bytes
		m_stamp = stamp; 
		m_status = status;
		m_note = note;
		m_velocity = velocity;
	
	}
	//destructor
	~MidiEvent()
	{}
	
	//mtof: convert from midi to frequency
	double mtof()
	{
		
		double freq;
		//base frequency: note 57: A3 220  Hz
		double base = 220.000000000;
		freq = base * pow(2, ((double)m_note-57.0)/12.0);
		return freq; 
	}
	
	int getChannel()
	{
		//4 LSBs
		return m_status%16;	
	}
	
	double getStamp()
	{
		return m_stamp;
	}
	int getStatus()
	{
		return m_status;
	}
	
	int getVelocity()
	{
		return m_velocity;
	}
	
	int getNote()
	{
		return m_note;
	}
	
private:
	
	int m_status; 
	int m_note;
	int m_velocity;
	//int channel ?
	double m_stamp;


};

#endif
